@@ -169,7 +169,7 @@ st_read.PostgreSQLConnection <- function(...) {
 }
 
 postgis_as_sfc <- function(x, EWKB, conn) {
-	geom <- st_as_sfc(as_wkb(x), EWKB = EWKB)
+	geom <- st_as_sfc(as_wkb_(x), EWKB = EWKB)
 	srid <- attr(geom, "srid")
 	if (!is.null(srid)) {
 		st_crs(geom) = db_find_srid(conn, srid = srid, validate = FALSE)
@@ -192,13 +192,13 @@ schema_table <- function(conn, table, public = "public") {
 	else if (length(table) > 2)
 		stop("table cannot be longer than 2 (schema, table)", call. = FALSE)
 
-	if (any(is.na(table)))
+	if (anyNA(table))
 		stop("table and schema cannot be NA", call. = FALSE)
 
 	return(table)
 }
 
-as_wkb <- function(x) {
+as_wkb_ <- function(x) {
 	structure(x, class = "WKB")
 }
 
@@ -242,7 +242,7 @@ db_find_srid = function(conn, crs_local = st_crs(srid), srid = epsg(crs_local),
     }
     crs_found <- st_crs(db_crs[["srtext"]])
     crs_found[["input"]] <- build_epsg(srid)
-    if(validate && crs_found != crs_local & !is.na(crs_local)) {
+    if(validate && crs_found != crs_local && !is.na(crs_local)) {
     	# TODO: pretty print db_spatial_ref
         warning("Local crs different from database crs. You can inspect the ",
                 "database crs using `dbReadtable(conn, \"spatial_ref_sys\")` ",
@@ -436,12 +436,12 @@ get_new_postgis_srid <- function(conn) {
 #' Write `sf` object to Database
 #' @inheritParams RPostgreSQL::postgresqlWriteTable
 #' @md
-#' @rdname st_write
+#' @rdname dbWriteTable
 #' @importMethodsFrom DBI dbWriteTable
 #' @export
 setMethod("dbWriteTable", c("PostgreSQLConnection", "character", "sf"),
           function(conn, name, value, ..., row.names = FALSE, overwrite = FALSE,
-                   append = FALSE, field.types = NULL, factorsAsCharacter = TRUE, binary = TRUE) {
+                   append = FALSE, field.types = NULL, binary = TRUE) {
               if (is.null(field.types)) field.types <- dbDataType(conn, value)
               tryCatch({
                   dbWriteTable(conn, name, to_postgis(conn, value, binary),..., row.names = row.names,
@@ -465,23 +465,28 @@ setMethod("dbWriteTable", c("PostgreSQLConnection", "character", "sf"),
 #' @param field.types default `NULL`. Allows to override type conversion from R
 #' to PostgreSQL. See `dbDataType()` for details.
 #' @md
-#' @rdname st_write
-#' @importMethodsFrom DBI dbWriteTable
+#' @rdname dbWriteTable
+#' @importMethodsFrom DBI dbWriteTable dbExistsTable
 #' @export
 setMethod("dbWriteTable", c("DBIObject", "character", "sf"),
-          function(conn, name, value, ..., row.names = FALSE, overwrite = FALSE,
-                   append = FALSE, field.types = NULL, factorsAsCharacter = TRUE, binary = TRUE) {
-          	if (is.null(field.types)) field.types <- dbDataType(conn, value)
-              # DBI cannot set field types with append
-              if (append) field.types <- NULL
-              #tryCatch({
-                  dbWriteTable(conn, name, to_postgis(conn, value, binary),..., row.names = row.names,
-                               overwrite = overwrite, append = append,
-                               field.types = field.types)
-              # }, warning=function(w) {
-              #     stop(conditionMessage(w), call. = FALSE)  # nocov
-              # })
-          }
+		  function(conn, name, value, ..., row.names = FALSE, overwrite = FALSE,
+		  		 append = FALSE, field.types = NULL, binary = TRUE) {
+		  	if (is.null(field.types)) field.types <- dbDataType(conn, value)
+
+		   	# DBI cannot set field types with append, but if the table does not exist,
+		  	# we need to set the field type.
+		  	if (append) {
+		  		if (!dbExistsTable(conn, name)) {
+		  			append <- FALSE
+		  		} else {
+		  			field.types <- NULL
+		  		}
+		  	}
+		  	
+		  	dbWriteTable(conn, name, to_postgis(conn, value, binary),..., row.names = row.names,
+		  				 overwrite = overwrite, append = append,
+		  				 field.types = field.types)
+		  }
 )
 
 to_postgis <- function(conn, x, binary) {
@@ -517,7 +522,6 @@ sync_crs <- function(conn, geom) {
 #' Determine database type for R vector
 #'
 #' @export
-#' @inheritParams RPostgreSQL dbDataType
 #' @rdname dbDataType
 #' @importMethodsFrom DBI dbDataType
 setMethod("dbDataType", c("PostgreSQLConnection", "sf"), function(dbObj, obj) {
@@ -533,7 +537,6 @@ setMethod("dbDataType", c("PostgreSQLConnection", "sf"), function(dbObj, obj) {
 #' Determine database type for R vector
 #'
 #' @export
-#' @inheritParams DBI dbDataType
 #' @rdname dbDataType
 #' @importClassesFrom DBI DBIObject
 #' @importMethodsFrom DBI dbDataType
@@ -556,10 +559,12 @@ setMethod("dbDataType", c("DBIObject", "sf"), function(dbObj, obj) {
 #' @param classes classes inherited
 is_geometry_column <- function(con, x, classes = "") UseMethod("is_geometry_column")
 
+#' @export
 is_geometry_column.PqConnection <- function(con, x, classes = c("pq_geometry")) {
     vapply(x, inherits, logical(1), classes)
 }
 
+#' @export
 is_geometry_column.default <- function(con, x, classes = c("character")) {
     # try all character columns (in conjunction with try_postgis_as_sfc)
     vapply(x, function(x) inherits(x, classes) && !all(is.na(x)),
