@@ -37,7 +37,7 @@ set_utf8 = function(x) {
 #' @param options character; driver dependent dataset open options, multiple
 #'   options supported. For possible values, see the "Open options" section
 #'   of the GDAL documentation of the corresponding driver, and
-#'   https://github.com/r-spatial/sf/issues/1157 for an example.
+#'   <https://github.com/r-spatial/sf/issues/1157> for an example.
 #' @param quiet logical; suppress info on name, driver, size and spatial
 #'   reference, or signaling no or multiple layers
 #' @param geometry_column integer or character; in case of multiple geometry
@@ -54,14 +54,14 @@ set_utf8 = function(x) {
 #'   converted to factors?  Default for \code{read_sf} or R version >= 4.1.0 is
 #' \code{FALSE}, for \code{st_read} and R version < 4.1.0 equal to
 #' \code{default.stringsAsFactors()}
-#' @param int64_as_string logical; if TRUE, Int64 attributes are returned as
-#'   string; if FALSE, they are returned as double and a warning is given when
+#' @param int64_as_string logical; if `TRUE`, Int64 attributes are returned as
+#'   string; if `FALSE`, they are returned as double and a warning is given when
 #'   precision is lost (i.e., values are larger than 2^53).
-#' @param check_ring_dir logical; if TRUE, polygon ring directions are checked
+#' @param check_ring_dir logical; if `TRUE`, polygon ring directions are checked
 #'   and if necessary corrected (when seen from above: exterior ring counter
 #'   clockwise, holes clockwise)
 #' @details for \code{geometry_column}, see also
-#' \url{https://trac.osgeo.org/gdal/wiki/rfc41_multiple_geometry_fields}
+#' \url{https://gdal.org/en/latest/development/rfc/rfc41_multiple_geometry_fields.html}
 #'
 #' for values for \code{type} see
 #' \url{https://en.wikipedia.org/wiki/Well-known_text#Well-known_binary}, but
@@ -79,13 +79,13 @@ set_utf8 = function(x) {
 #' data sources) directly from zip files can be done by prepending the path 
 #' with \code{/vsizip/}. This is part of the GDAL Virtual File Systems interface
 #' that also supports .gz, curl, and other operations, including chaining; see
-#' \url{https://gdal.org/user/virtual_file_systems.html} for a complete
+#' \url{https://gdal.org/en/latest/user/virtual_file_systems.html} for a complete
 #' description and examples.
 #'
 #' For \code{query} with a character \code{dsn} the query text is handed to
 #' 'ExecuteSQL' on the GDAL/OGR data set and will result in the creation of a
 #' new layer (and \code{layer} is ignored). See 'OGRSQL'
-#' \url{https://gdal.org/user/ogr_sql_dialect.html} for details. Please note that the
+#' \url{https://gdal.org/en/latest/user/ogr_sql_dialect.html} for details. Please note that the
 #' 'FID' special field is driver-dependent, and may be either 0-based (e.g. ESRI
 #' Shapefile), 1-based (e.g. MapInfo) or arbitrary (e.g. OSM). Other features of
 #' OGRSQL are also likely to be driver dependent. The available layer names may
@@ -269,7 +269,7 @@ process_cpl_read_ogr_stream = function(x, geom_column_info, num_features, fid_co
 #' @param drivers character; limited set of driver short names to be tried (default: try all)
 #' @param wkt_filter character; WKT representation of a spatial filter (may be used as bounding box, selecting overlapping geometries); see examples
 #' @param optional logical; passed to \link[base]{as.data.frame}; always \code{TRUE} when \code{as_tibble} is \code{TRUE}
-#' @param use_stream Use TRUE to use the experimental columnar interface introduced in GDAL 3.6.
+#' @param use_stream Use `TRUE` to use the experimental columnar interface introduced in GDAL 3.6.
 #' @note The use of \code{system.file} in examples make sure that examples run regardless where R is installed:
 #' typical users will not use \code{system.file} but give the file name directly, either with full path or relative
 #' to the current working directory (see \link{getwd}). "Shapefiles" consist of several files with the same basename
@@ -295,8 +295,6 @@ st_read.character = function(dsn, layer, ..., query = NA, options = NULL, quiet
 	if (length(promote_to_multi) > 1)
 		stop("`promote_to_multi' should have length one, and applies to all geometry columns")
 
-
-
 	if (use_stream) {
 		stream = nanoarrow::nanoarrow_allocate_array_stream()
 		info = CPL_read_gdal_stream(stream, dsn, layer, query, as.character(options), quiet,
@@ -405,8 +403,8 @@ abbreviate_shapefile_names = function(x) {
 #'   is missing, the \code{basename} of \code{dsn} is taken.
 #' @param driver character; name of driver to be used; if missing and \code{dsn} is not a Database Connection, a driver name is guessed from \code{dsn};
 #' \code{st_drivers()} returns the drivers that are available with their properties; links to full driver documentation
-#' are found at \url{https://gdal.org/drivers/vector/index.html}
-#' @param ... other arguments passed to \link{dbWriteTable} when \code{dsn} is a
+#' are found at \url{https://gdal.org/en/latest/drivers/vector/index.html}
+#' @param ... other arguments passed to \link[DBI]{dbWriteTable} when \code{dsn} is a
 #' Database Connection
 #' @param dataset_options character; driver dependent dataset creation options;
 #' multiple options supported.
@@ -443,7 +441,7 @@ abbreviate_shapefile_names = function(x) {
 #' When deleting layers or data sources is not successful, no error is emitted.
 #' \code{delete_dsn} and \code{delete_layer} should be
 #' handled with care; the former may erase complete directories or databases.
-#' @seealso \link{st_drivers}, \link{dbWriteTable}
+#' @seealso \link{st_drivers}, \link[DBI]{dbWriteTable}
 #' @return \code{obj}, invisibly
 #' @examples
 #' nc = st_read(system.file("shape/nc.shp", package="sf"))
@@ -625,7 +623,7 @@ write_sf <- function(..., quiet = TRUE, append = FALSE, delete_layer = !append)
 #'   readable, and which may be written. The field `vsi` refers to the driver's
 #'   capability to read/create datasets through the VSI*L API. [See GDAL website
 #'   for additional details on driver
-#'   support](https://gdal.org/drivers/vector/index.html).
+#'   support](https://gdal.org/en/latest/drivers/vector/index.html)
 #' @return A `data.frame` with driver metadata.
 #' @export
 #' @md
@@ -700,7 +698,7 @@ st_layers = function(dsn, options = character(0), do_count = FALSE) {
 		Encoding(ret[[1]]) <- "UTF-8"
 		ret[[1]] <- enc2native(ret[[1]])
 	}
-	ret
+	structure(ret, row.names = seq_along(ret[[1]]), class = c("sf_layers", "data.frame"))
 }
 
 guess_driver = function(dsn) {
@@ -826,8 +824,8 @@ check_append_delete <- function(append, delete) {
 
 #' @name st_write
 #' @export
-#' @details st_delete deletes layer(s) in a data source, or a data source if layers are
-#' omitted; it returns TRUE on success, FALSE on failure, invisibly.
+#' @details `st_delete()` deletes layer(s) in a data source, or a data source if layers are
+#' omitted; it returns `TRUE` on success, `FALSE` on failure, invisibly.
 st_delete = function(dsn, layer = character(0), driver = guess_driver_can_write(dsn), quiet = FALSE) {
 	invisible(CPL_delete_ogr(dsn, layer, driver, quiet) == 0)
 }
