@@ -340,9 +340,10 @@ gdal_rasterize = function(sf, x, gt, file, driver = "GTiff", options = character
 #' @rdname gdal
 #' @param f gdal raster data source filename
 #' @param pts points matrix
-#' @param bilinear logical; use bilinear interpolation, rather than nearest neighbor?
-gdal_extract = function(f, pts, bilinear = FALSE) {
-	CPL_extract(f, pts, as.logical(bilinear))
+#' @param resampling character; resampling method; for method cubic or cubicspline, 
+#' `stars_proxy` objects should be used and GDAL should have version >= 3.10.0
+gdal_extract = function(f, pts, resampling = c("nearest", "bilinear", "cubic", "cubicspline")) {
+	CPL_extract(f, pts, match.arg(resampling))
 }
 
 #' @rdname gdal
@@ -354,7 +355,7 @@ gdal_extract = function(f, pts, bilinear = FALSE) {
 #' @param proxy logical; return proxy object?
 #' @param debug logical; print debug messages?
 #' @export
-gdal_read_mdim = function(file, array_name = character(0), options = character(0), 
+gdal_read_mdim = function(file, array_name = character(0), options = character(0),
 						  offset = integer(0), count = integer(0), step = integer(0), 
 						  proxy = FALSE, debug = FALSE) {
 	CPL_read_mdim(file, array_name, options, offset, count, step, proxy, debug)
@@ -388,7 +389,7 @@ gdal_create = function(f, nxy, values, crs, xlim, ylim) {
 	CPL_create(as.character(f), as.integer(nxy), as.double(values), crs$wkt, as.double(xlim), as.double(ylim))
 }
 
-#' add or remove overviews to/from a raster image
+#' Add or remove overviews to/from a raster image
 #'
 #' add or remove overviews to/from a raster image
 #' @param file character; file name
@@ -404,7 +405,7 @@ gdal_create = function(f, nxy, values, crs, xlim, ylim) {
 #' @export
 gdal_addo = function(file, overviews = c(2,4,8,16), method = "NEAREST", layers = integer(0), 
 					 options = character(0), config_options = character(0), clean = FALSE, read_only = FALSE) {
-	stopifnot(length(method) == 1, is.character(method), is.numeric(overviews))
+	stopifnot(length(method) == 1, is.character(method), is.numeric(overviews), is.character(config_options))
 	invisible(CPL_gdaladdo(file, method, as.integer(overviews), as.integer(layers), as.character(options), 
-				 as.character(config_options), as.logical(clean)[1], as.logical(read_only)[1]))
+				 config_options, as.logical(clean)[1], as.logical(read_only)[1]))
 }
