@@ -1,10 +1,3 @@
-## ----echo=FALSE, include=FALSE------------------------------------------------
-knitr::opts_chunk$set(fig.height = 4.5)
-knitr::opts_chunk$set(fig.width = 6)
-tmap_fixed = TRUE
-#user_edzer_or_travis = Sys.getenv("USER") %in% c("travis", "edzer")
-user_edzer_or_travis = FALSE
-
 ## -----------------------------------------------------------------------------
 library(sf)
 demo(nc, ask = FALSE, echo = FALSE)
@@ -36,7 +29,7 @@ plot(nc["AREA"], key.pos = 1, axes = TRUE, key.width = lcm(1.3), key.length = 1.
 
 ## -----------------------------------------------------------------------------
 nc$f = cut(nc$AREA, 10)
-plot(nc["f"], axes = TRUE, key.pos = 4, pal = sf.colors(10), key.width = lcm(4.5))
+plot(nc["f"], axes = TRUE, key.pos = 4, pal = sf.colors(10), key.width = lcm(5))
 
 ## -----------------------------------------------------------------------------
 plot(nc["AREA"], breaks = c(0,.05,.1,.15,.2,.25))
@@ -90,20 +83,20 @@ ggplot() +
   facet_wrap(~VAR, ncol = 1) +
   scale_y_continuous(breaks = 34:36)
 
-## ----eval=user_edzer_or_travis------------------------------------------------
+## ----eval = FALSE-------------------------------------------------------------
 #  library(mapview)
-#  mapviewOptions(fgb = FALSE)
+#  mapviewOptions(fgb = FALSE) # needed when creating web pages
 #  mapview(nc["BIR74"], col.regions = sf.colors(10), fgb = FALSE)
 
-## -----------------------------------------------------------------------------
+## ----eval=require("tmap", quietly = TRUE)-------------------------------------
 library(tmap)
 qtm(nc)
 
-## ----eval=tmap_fixed----------------------------------------------------------
-tmap_mode("view")
-tm_shape(nc) + tm_fill("BIR74", palette = sf.colors(5))
+## ----eval=FALSE---------------------------------------------------------------
+#  tmap_mode("view")
+#  tm_shape(nc) + tm_fill("BIR74", palette = sf.colors(5))
 
-## ----eval=tmap_fixed----------------------------------------------------------
-ttm()
-tmap_last()
+## ----eval=FALSE---------------------------------------------------------------
+#  ttm()
+#  tmap_last()
 
