@@ -35,6 +35,9 @@
 # if GEOS_VERSION_MINOR >= 11
 #  define HAVE311
 # endif
+# if GEOS_VERSION_MINOR >= 12
+#  define HAVE312
+# endif
 #else
 # if GEOS_VERSION_MAJOR > 3
 #  define HAVE340
@@ -1376,3 +1379,41 @@ Rcpp::List CPL_nary_intersection(Rcpp::List sfc) {
 	// return result
 	return ret;
 }
+
+// [[Rcpp::export]]
+Rcpp::NumericVector CPL_line_project(Rcpp::List lines, Rcpp::List points, bool normalized) {
+	GEOSContextHandle_t hGEOSCtxt = CPL_geos_init();
+	int dim = 2;
+	std::vector<GeomPtr> l = geometries_from_sfc(hGEOSCtxt, lines, &dim);
+	std::vector<GeomPtr> p = geometries_from_sfc(hGEOSCtxt, points, &dim);
+	Rcpp::NumericVector ret(p.size());
+	if (normalized) {
+		for (size_t i = 0; i < l.size() && i < p.size(); i++)
+			ret[i] = GEOSProjectNormalized_r(hGEOSCtxt, l[i].get(), p[i].get());
+	} else {
+		for (size_t i = 0; i < l.size() && i < p.size(); i++)
+			ret[i] = GEOSProject_r(hGEOSCtxt, l[i].get(), p[i].get());
+	}
+	CPL_geos_finish(hGEOSCtxt);
+	// return result
+	return ret;
+}
+
+// [[Rcpp::export]]
+Rcpp::List CPL_line_interpolate(Rcpp::List lines, Rcpp::NumericVector dists, bool normalized) {
+	GEOSContextHandle_t hGEOSCtxt = CPL_geos_init();
+	int dim = 2;
+	std::vector<GeomPtr> l = geometries_from_sfc(hGEOSCtxt, lines, &dim);
+	std::vector<GeomPtr> p(l.size());
+	if (normalized) {
+		for (int i = 0; i < (int) l.size() && i < dists.size(); i++)
+			p[i] = geos_ptr(GEOSInterpolateNormalized_r(hGEOSCtxt, l[i].get(), dists[i]), hGEOSCtxt);
+	} else {
+		for (int i = 0; i < (int) l.size() && i < dists.size(); i++)
+			p[i] = geos_ptr(GEOSInterpolate_r(hGEOSCtxt, l[i].get(), dists[i]), hGEOSCtxt);
+	}
+	Rcpp::List ret(sfc_from_geometry(hGEOSCtxt, p, dim));
+	CPL_geos_finish(hGEOSCtxt);
+	// return result
+	return ret;
+}
