@@ -284,8 +284,9 @@ List get_rat(GDALRasterAttributeTable *tbl) {
 
 // [[Rcpp::export]]
 List CPL_read_gdal(CharacterVector fname, CharacterVector options, CharacterVector driver,
-		bool read_data, NumericVector NA_value, List RasterIO_parameters) {
-// reads and returns data set metadata, and if read_data is true, adds data array
+		bool read_data, NumericVector NA_value, List RasterIO_parameters, double max_cells) {
+// reads and returns data set metadata, and adds data array if read_data is true, or less 
+// than max_cells are to be read
 	GDALDataset *poDataset = (GDALDataset *) GDALOpenEx(fname[0], GA_ReadOnly,
 		driver.size() ? create_options(driver).data() : NULL,
 		options.size() ? create_options(options).data() : NULL,
@@ -323,7 +324,7 @@ List CPL_read_gdal(CharacterVector fname, CharacterVector options, CharacterVect
 	bool geo_transform_set = (err == CE_None);
 
 	// CRS, projection:
-#if GDAL_VERSION_MAJOR >= 3
+#if GDAL_VERSION_NUM >= 3000000
 	const OGRSpatialReference *sr = poDataset->GetSpatialRef();
 	// sr = handle_axis_order(sr); -- should be done by GDAL; xy
 	Rcpp::List crs = create_crs(sr, true);
@@ -365,6 +366,7 @@ List CPL_read_gdal(CharacterVector fname, CharacterVector options, CharacterVect
 	CharacterVector descriptions(bands.size());
 	NumericMatrix ranges(bands.size(), 4);
 	IntegerMatrix blocksizes(bands.size(), 2);
+	IntegerVector colorInterp(bands.size());
 	for (int i = 0; i < bands.size(); i++) {
 		if ((poBand = poDataset->GetRasterBand(bands(i))) == NULL)
 			stop("trying to read a band that is not present");
@@ -390,6 +392,7 @@ List CPL_read_gdal(CharacterVector fname, CharacterVector options, CharacterVect
 		poBand->GetBlockSize(&nBlockXSize, &nBlockYSize);
 		blocksizes(i, 0) = nBlockXSize;
 		blocksizes(i, 1) = nBlockYSize;
+		colorInterp(i) = (int) poBand->GetColorInterpretation();
 	}
 
 	// get metadata items:
@@ -409,6 +412,9 @@ List CPL_read_gdal(CharacterVector fname, CharacterVector options, CharacterVect
 	int nBufXSize = get_from_list(RasterIO_parameters, "nBufXSize", nXSize);
 	int nBufYSize = get_from_list(RasterIO_parameters, "nBufYSize", nYSize);
 
+	if (max_cells > 0) 
+		read_data = (bands.size() * nBufXSize * nBufYSize) < max_cells;
+
 	// resampling method:
 	GDALRasterIOExtraArg resample;
 	INIT_RASTERIO_EXTRA_ARG(resample);
@@ -440,7 +446,7 @@ List CPL_read_gdal(CharacterVector fname, CharacterVector options, CharacterVect
 		_["cols"] = NumericVector::create(nXOff + 1, nXOff + nBufXSize),
 		_["rows"] = NumericVector::create(nYOff + 1, nYOff + nBufYSize),
 		_["bands"] = bands,
-#if GDAL_VERSION_MAJOR >= 3
+#if GDAL_VERSION_NUM >= 3000000
 		_["crs"] = crs,
 #else
 		_["proj_wkt"] = wkt,
@@ -457,7 +463,9 @@ List CPL_read_gdal(CharacterVector fname, CharacterVector options, CharacterVect
 		_["ranges"] = ranges,
 		_["blocksizes"] = blocksizes,
 		_["descriptions"] = descriptions,
-		_["default_geotransform"] = default_geotransform
+		_["default_geotransform"] = default_geotransform,
+		_["proxy"] = LogicalVector::create(!read_data),
+		_["colorInterp"] = colorInterp
 	);
 	if (read_data) {
 		ReturnList.attr("data") = read_gdal_data(poDataset, nodatavalue, nXOff, nYOff,
@@ -486,7 +494,7 @@ List CPL_read_gdal(CharacterVector fname, CharacterVector options, CharacterVect
 // [[Rcpp::export]]
 void CPL_write_gdal(NumericMatrix x, CharacterVector fname, CharacterVector driver,
 		CharacterVector options, CharacterVector Type, IntegerVector dims, IntegerVector from,
-		NumericVector gt, CharacterVector p4s, NumericVector na_val,
+		NumericVector gt, CharacterVector p4s, NumericVector na_val, NumericVector scale_offset,
 		bool create = true, bool only_create = false) {
 
 	// figure out driver:
@@ -498,8 +506,8 @@ void CPL_write_gdal(NumericMatrix x, CharacterVector fname, CharacterVector driv
 
 	// can this driver Create()?
 	char **papszMetadata = poDriver->GetMetadata();
-	if (!CSLFetchBoolean( papszMetadata, GDAL_DCAP_CREATE, FALSE ) )
-		stop("driver does not support Create() method."); // #nocov
+	if (!CSLFetchBoolean( papszMetadata, GDAL_DCAP_CREATE, FALSE ) && !CSLFetchBoolean( papszMetadata, GDAL_DCAP_CREATECOPY, FALSE ) )
+		stop("driver does not support Create() or CreateCopy() method."); // #nocov
 
 	// figure out eType:
 	GDALDataType eType = GDT_Unknown;
@@ -507,6 +515,10 @@ void CPL_write_gdal(NumericMatrix x, CharacterVector fname, CharacterVector driv
 		stop("Type should have length 1"); // #nocov
 	if (Type[0] == "Byte")
 		eType = GDT_Byte; // #nocov
+#if GDAL_VERSION_NUM >= 3070000
+	else if (Type[0] == "Int8")
+		eType = GDT_Int8; // #nocov
+#endif
 	else if (Type[0] == "UInt16")
 		eType = GDT_UInt16; // #nocov
 	else if (Type[0] == "Int16")
@@ -531,13 +543,30 @@ void CPL_write_gdal(NumericMatrix x, CharacterVector fname, CharacterVector driv
 		stop("from should have length two"); // #nocov
 	if (na_val.length() != 1)
 		stop("na_val should have length 1"); // #nocov
+	if (scale_offset.size() != 2)
+		stop("scale_offset should have length 2");
+	bool set_scale_offset = (scale_offset[0] != 1.0 || scale_offset[1] != 0.0);
 
 	// create dataset:
 	GDALDataset *poDstDS;
 	if (create) {
 		if (from[0] != 0 || from[1] != 0)
 			stop("from values should be zero when creating a dataset"); // #nocov
-		if ((poDstDS = poDriver->Create( fname[0], dims[0], dims[1], dims[2], eType,
+
+		if (poDriver->GetMetadataItem(GDAL_DCAP_CREATE) == NULL && 
+						poDriver->GetMetadataItem(GDAL_DCAP_CREATECOPY) != NULL) {
+			GDALDriver *memDriver = GetGDALDriverManager()->GetDriverByName("MEM");
+			GDALDataset *memDS;
+			if ((memDS = memDriver->Create(fname[0], dims[0], dims[1], dims[2], eType, NULL)) == NULL)
+				stop("cannot create copy in memory"); // #nocov
+			options.push_back("APPEND_SUBDATASET=YES");
+			if ((poDstDS = poDriver->CreateCopy(fname[0], memDS, FALSE, 
+										create_options(options).data(), NULL, NULL)) == NULL) {
+				GDALClose(memDS);
+				stop("cannot CreateCopy from memory dataset");
+			} else
+				GDALClose(memDS);
+		} else if ((poDstDS = poDriver->Create( fname[0], dims[0], dims[1], dims[2], eType,
 				create_options(options).data())) == NULL)
 			stop("creating dataset failed"); // #nocov
 
@@ -556,7 +585,7 @@ void CPL_write_gdal(NumericMatrix x, CharacterVector fname, CharacterVector driv
 		if (p4s[0] != NA_STRING) {
 			OGRSpatialReference oSRS;
 			oSRS.SetFromUserInput((const char *) p4s[0]); // handles wkt too
-#if GDAL_VERSION_MAJOR >= 3
+#if GDAL_VERSION_NUM >= 3000000
 			poDstDS->SetSpatialRef(&oSRS);
 #else
 			char *pszSRS_WKT = NULL;
@@ -577,6 +606,15 @@ void CPL_write_gdal(NumericMatrix x, CharacterVector fname, CharacterVector driv
 				}
 			}
 		}
+		// set scale_offset
+		if (set_scale_offset) {
+			for (int band = 1; band <= dims(2); band++) { // unlike x & y, band is 1-based
+				GDALRasterBand *poBand = poDstDS->GetRasterBand( band );
+				if (poBand->SetScale(scale_offset[0]) != CE_None ||
+						poBand->SetOffset(scale_offset[1]) != CE_None)
+					warning("writing scale and/or offset failed (not supported by driver)"); // #nocov
+			}
+		}
 		// set descriptions:
 		if (x.attr("descriptions") != R_NilValue) {
 			Rcpp::CharacterVector descriptions = x.attr("descriptions");
@@ -589,13 +627,9 @@ void CPL_write_gdal(NumericMatrix x, CharacterVector fname, CharacterVector driv
 		// write factor levels to CategoryNames:
 		if (x.attr("levels") != R_NilValue) {
 			Rcpp::CharacterVector levels = x.attr("levels");
-			Rcpp::CharacterVector l(levels.size() + 1);
-			l[0] = ""; // levels start at 1, CategoryNames at 0.
-			for (int i = 0; i < levels.size(); i++)
-				l[i+1] = levels[i];
 			for (int band = 1; band <= dims(2); band++) {
 				GDALRasterBand *poBand = poDstDS->GetRasterBand( band );
-				if (poBand->SetCategoryNames(create_options(l).data()) != CE_None)
+				if (poBand->SetCategoryNames(create_options(levels).data()) != CE_None)
 					warning("error writing factor levels to raster band");
 			}
 		}
@@ -604,14 +638,12 @@ void CPL_write_gdal(NumericMatrix x, CharacterVector fname, CharacterVector driv
 			Rcpp::NumericMatrix co = x.attr("rgba"); // r g b alpha in columns; levels in rows
 			GDALColorTable ct = GDALColorTable(GPI_RGB);
 			GDALColorEntry ce;
-			ce.c1 = 0; ce.c2 = 0; ce.c3 = 0; ce.c4 = 0;
-			ct.SetColorEntry(0, &ce);
 			for (int i = 0; i < co.nrow(); i++) {
 				ce.c1 = co(i, 0);
 				ce.c2 = co(i, 1);
 				ce.c3 = co(i, 2);
 				ce.c4 = co(i, 3);
-				ct.SetColorEntry(i + 1, &ce);
+				ct.SetColorEntry(i, &ce);
 			}
 			GDALRasterBand *poBand = poDstDS->GetRasterBand( 1 ); // can only set CT for band 1
 			if (poBand->SetColorTable(&ct) != CE_None)
@@ -635,6 +667,14 @@ void CPL_write_gdal(NumericMatrix x, CharacterVector fname, CharacterVector driv
 			}
 			checkUserInterrupt();
 		}
+		if (set_scale_offset) {
+			for (int i = 0; i < x.ncol(); i++) {
+				NumericVector nv = x(_, i);
+				for (int j = 0; j < x.nrow(); j++)
+					nv[j] = (nv[j] - scale_offset[1]) / scale_offset[0]; // raw = (units - offset) / scale
+				x(_, i) = nv;
+			}
+		}
 		// write values:
 		// write the whole lot:
 		if (poDstDS->RasterIO(GF_Write, from[0], from[1], dims[0] - from[0], dims[1] - from[1],
@@ -655,35 +695,42 @@ double get_bilinear(GDALRasterBand *poBand, double Pixel, double Line,
 	double pixels[4];
 	double dY  = Line - iLine; // [0, 1) over a raster cell
 	double dX  = Pixel - iPixel; // [0, 1) over a raster cell
-	if ((dY < 0.5 && iLine > 0) || (iLine == RasterYSize - 1)) // where to start reading
+	double eps = 1.0e-13;
+	if ((dY < 0.5 && iLine > 0) || (iLine == RasterYSize - 1)) { // where to start reading
 		iLine -= 1;
-	if ((dX < 0.5 && iPixel > 0) || (iPixel == RasterXSize - 1))
+		dY += 1.0;
+	}
+	if ((dX < 0.5 && iPixel > 0) || (iPixel == RasterXSize - 1)) {
 		iPixel -= 1;
+		dX += 1.0;
+	}
 
 	// x:
-	if (Pixel < 0.5) // border:
+	if (Pixel < (0.5 - eps)) // border:
 		dX = 0.0;
-	else if (Pixel > RasterXSize - 0.5)
+	else if (Pixel > (RasterXSize - 0.5 + eps))
 		dX = 1.0;
-	else if (dX < 0.5) // shift to pixel center:
+	else if (dX < (0.5 - eps)) // shift to pixel center:
 		dX += 0.5;
 	else
 		dX -= 0.5;
 
 	// y:
-	if (Line < 0.5)
+	if (Line < (0.5 - eps))
 		dY = 0.0;
-	else if (Line > RasterYSize - 0.5)
+	else if (Line > (RasterYSize - 0.5 + eps))
 		dY = 1.0;
-	else if (dY < 0.5)
+	else if (dY < (0.5 - eps))
 		dY += 0.5;
 	else
 		dY -= 0.5;
 
 	// read:
-	if (GDALRasterIO(poBand, GF_Read, iPixel, iLine, 2, 2,
-			(void *) pixels, 2, 2, GDT_CFloat64, sizeof(double), 0) != CE_None)
+	if (poBand->RasterIO(GF_Read, iPixel, iLine, 2, 2,
+			(void *) pixels, 2, 2, GDT_CFloat64, sizeof(double), 0, NULL) != CE_None)
 		stop("Error reading!");
+	// Rprintf("px[%g (%g) %g (%g) %g (%g) %g (%g)] dY: %g dX: %g iLine: %d iPixel: %d Line: %g Pixel :%g\n", pixels[0], (1-dX)*(1-dY), pixels[1], dX * (1-dY), pixels[2], (1-dX) * dY, pixels[3], dX * dY, dY, dX, iLine, iPixel, Line, Pixel);
+	//
 	// f(0,0): pixels[0], f(1,0): pixels[1], f(0,1): pixels[2], f(1,1): pixels[3]
 	if (na_set && (pixels[0] == na_value || pixels[1] == na_value ||
 			pixels[2] == na_value || pixels[3] == na_value))
@@ -696,7 +743,7 @@ double get_bilinear(GDALRasterBand *poBand, double Pixel, double Line,
 }
 
 // [[Rcpp::export]]
-NumericMatrix CPL_extract(CharacterVector input, NumericMatrix xy, bool interpolate = false) {
+NumericMatrix CPL_extract(CharacterVector input, NumericMatrix xy, CharacterVector interpolate) {
 	// mostly taken from gdal/apps/gdallocationinfo.cpp
 
 	GDALDataset *poDataset = (GDALDataset *) GDALOpenEx(input[0], GA_ReadOnly,
@@ -709,6 +756,17 @@ NumericMatrix CPL_extract(CharacterVector input, NumericMatrix xy, bool interpol
 	NumericMatrix ret(xy.nrow(), poDataset->GetRasterCount());
 	int xsize = poDataset->GetRasterXSize();
 	int ysize = poDataset->GetRasterYSize();
+	GDALRIOResampleAlg RA;
+	if (interpolate[0] == "nearest")
+		RA = GRIORA_NearestNeighbour;
+	else if (interpolate[0] == "bilinear")
+		RA = GRIORA_Bilinear;
+	else if (interpolate[0] == "cubic")
+		RA = GRIORA_Cubic;
+	else if (interpolate[0] == "cubicspline")
+		RA = GRIORA_CubicSpline;
+	else
+		stop("interpolation method not supported"); // #nocov
 
 	double gt[6];
 	poDataset->GetGeoTransform(gt);
@@ -738,13 +796,20 @@ NumericMatrix CPL_extract(CharacterVector input, NumericMatrix xy, bool interpol
 			if (iPixel < 0 || iLine < 0 || iPixel >= xsize || iLine >= ysize) // outside bbox:
 				pixel = NA_REAL;
 			else { // read pixel:
-				if (interpolate)
-					// stop("interpolate not implemented");
+#if GDAL_VERSION_NUM >= 3100000
+				if (poBand->InterpolateAtPoint(Pixel, Line, RA, &pixel, nullptr) != CE_None)
+						// tbd: handle GRIORA_Cubic, GRIORA_CubicSpline
+					stop("Error in InterpolateAtPoint()");
+#else
+				if (RA == GRIORA_Cubic || RA == GRIORA_CubicSpline)
+					stop("cubic or cubicspline requires GDAL >= 3.10.0");
+				if (RA == GRIORA_Bilinear)
 					pixel = get_bilinear(poBand, Pixel, Line, iPixel, iLine,
 						xsize, ysize, nodata_set, nodata);
-				else if (GDALRasterIO(poBand, GF_Read, iPixel, iLine, 1, 1,
-						&pixel, 1, 1, GDT_CFloat64, 0, 0) != CE_None)
+				else if (poBand->RasterIO(GF_Read, iPixel, iLine, 1, 1,
+						&pixel, 1, 1, GDT_CFloat64, 0, 0, NULL) != CE_None)
 					stop("Error reading!");
+#endif
 				if (nodata_set && pixel == nodata)
 					pixel = NA_REAL;
 				else if (dfOffset != 0.0 || dfScale != 1.0)
@@ -758,7 +823,7 @@ NumericMatrix CPL_extract(CharacterVector input, NumericMatrix xy, bool interpol
 }
 
 // [[Rcpp::export]]
-void CPL_create(CharacterVector file, IntegerVector nxy, NumericVector value, CharacterVector wkt, 
+void CPL_create(CharacterVector file, IntegerVector nxy, NumericVector value, CharacterVector wkt,
 				NumericVector xlim, NumericVector ylim) {
 //
 // modified from gdal/apps/gdal_create.cpp:
@@ -768,7 +833,9 @@ void CPL_create(CharacterVector file, IntegerVector nxy, NumericVector value, Ch
                                    file[0], nPixels, nLines,
                                    1, GDT_Byte, NULL);
 	OGRSpatialReference oSRS;
+#if GDAL_VERSION_NUM >= 2050000
 	oSRS.SetAxisMappingStrategy(OAMS_TRADITIONAL_GIS_ORDER);
+#endif
 	if (oSRS.SetFromUserInput( wkt[0] ) != OGRERR_NONE) {
 		CPLError(CE_Failure, CPLE_AppDefined, "Failed to process SRS definition");
 		stop("CPL_create failed");
